@@ -1022,6 +1022,12 @@ class OKULARCORE_EXPORT Document : public QObject
         void reloadDocument() const;
 
     Q_SIGNALS:
+        /**
+         * This signal is emitted whenever the document is about to close.
+         * @since 1.5.3
+         */
+        void aboutToClose();
+
         /**
          * This signal is emitted whenever an action requests a
          * document close operation.
